import { Request as ExpressRequest, Response as ExpressResponse } from "express";
import { OAuth2 } from "src/utils/util";
import { ENVIRONMENT } from "src/utils/environment";
import { logger } from "src/utils/logging";

interface CodeResponse {
    code?: string;
}

export const get = async (req: ExpressRequest, res: ExpressResponse): Promise<void> => {
    const data = req.query as CodeResponse;
    if (!data.code) {
        res.redirect("/");
        return;
    }
    const result = await OAuth2.getToken({
        client_id: ENVIRONMENT.clientId,
        client_secret: ENVIRONMENT.clientSecret,
        grant_type: "authorization_code",
        code: data.code,
        scope: "identify",
        redirect_uri: ENVIRONMENT.redirectUrl,
    });
    if (result.ok) {
        logger.info("Got result: %s", result);
    }
    res.redirect("/");
};
